
proc bringup_table { db table } { 
    if { [catch {dblookup $db 0 $table 0 0} error]} { 
	tkdialog .a Error $error Ok
    } else { 
	show_table $error
	}
    }

proc set_menu_tables { db m } { 
    $m delete 0 last
    set new ""
    set tables [dbquery $db dbSCHEMA_TABLES]
    foreach table $tables {
	set db [dblookup $db 0 $table 0 0]
	if { ! [catch {dbquery $db dbRECORD_COUNT} count] && $count > 0 } { 
	    lappend new $table
	}
    }
    foreach table [lsort $new] {
	$m add command -label $table -command "bringup_table \"$db\" $table"
    }
}

proc startnewtable { w db table } {
    destroy $w
    newtable $db $table
}

proc select_table { db tables } { 
    set n [llength $tables] 
    set m [expr int(sqrt($n))]
    set w .select_table
    toplevel $w
    set row 0
    set col 0
    set max 0 
    set tables [split $tables]
    foreach table $tables { 
	set l [string length $table]
	if { $l > $max } {
	    set max $l
	}
    }
    foreach table $tables { 
	button $w.b_$table -text $table -width $max -command "startnewtable $w {$db} $table"
	grid $w.b_$table -row $row -column $col -sticky ew
	incr col
	if { $col > $m } { 
	    set col 0
	    incr row
	}
    }
    incr row
    set col 0
    button $w.cancel -text cancel -fg red -command "destroy $w"
    grid $w.cancel -row $row -column $col -columnspan [expr $m+1] -sticky ew
    raise $w
}


proc set_menu_newtables { db m } { 
    $m delete 0 last
    set new ""
    set tables [dbquery $db dbSCHEMA_TABLES]
    foreach table $tables { 
	set db [dblookup $db 0 $table 0 0]
	if { [lindex $db 1] >= 0 } { 
	    if { [catch {dbquery $db dbTABLE_IS_VIEW} errors] } { 
		puts stderr "dbquery ($db) dbTABLE_IS_VIEW returns $errors"
		break 
	    }
	    if {[catch {dbquery $db dbRECORD_COUNT} count] } { 
		;
	    } elseif { $count == 0 } { 
		lappend new $table
	    }
	}
    }
    set new [lsort $new]
    set ntables [llength $new]
    if { $ntables > 10 } { 
	$m add command -label "Select from grid" -command "select_table {$db} {$new}"
    } else { 
	foreach table $new {
	    $m add command -label $table -command "newtable \"$db\" $table"
	}
    }
}

proc set_Tables { db } { 
    global Tables 
    set tables_list [lsort [tables $db]] 
    set database [lindex $db 0]
    set Tables($database) ""
    foreach table $tables_list {
	set dbt [dblookup $db 0 $table 0 0]
	if { ! [catch {dbquery $dbt dbRECORD_COUNT} n] && $n > 0 } { 
	    lappend Tables($database) $table
	}
    }
}

proc set_table_buttons { db } { 
    global Tables Window Maxwidth
    set database [lindex $db 0]
    set w $Window($database)
    if { [winfo exists $w] } { 
	set slaves [table search $w table*]
	if { [clength $slaves] > 0 } { 
	    table forget $slaves
	}

	set col 0
	set maxcol 10
	set firstrow 10
	set row $firstrow
	set width 0
	foreach table $Tables($database) {
	    set b $w.show$table
	    if { ! [winfo exists $b] } { 
		button $b -text $table -command  "bringup_table \"$db\" $table"
	    }
	    set width [expr $width+[winfo reqwidth $b]]
	    if { $row == $firstrow && $width > $Maxwidth(.)-100 } {
		set maxcol [expr $col-1]
		set col 0
		incr row
	    } elseif { $col > $maxcol } {
		set col 0
		incr row
	    }
	    table $w $b $row,$col -anchor center -fill x
	    incr col
	}
    }
}

    


#
# Put up the initial table selection window
#
proc database_show { database_name args } { 
    global Pf Defaults Window Schema

    if { [catch {set db [dbopen_database $database_name r+]} error] } { 
	tkdialog .a Error $error Ok
	return 
    }
    set database [lindex $db 0]

    if { [lindex $db 1] < 0 } {
	if { [info exists Window($database)] } {
	    raise $Window($database)
	} else { 
	    set schema [dbquery $db dbSCHEMA_NAME]
	    if { [info exists Schema] } {
		if { $schema != $Schema } { 
		    puts stderr "All databases from the command line must share the same schema."
		    puts stderr "  This command line has both $Schema and $schema"
		    puts stderr "  Please run dbe separately on each database."
		    exit
		}
	    } else { 
		set Schema $schema 
	    }

	    catch "pfgetarr Defaults %${Pf}#$schema"
	    set_Tables $db
	    set w [uniqueW ""]
	    set Window($database) $w

	    set w $Window($database)
	    toplevel $w 

	    titleicon $w $db 

	    global icon_path
	    wm iconbitmap $w @$icon_path/dbicon
	    
	    set mb [menubar $w $db spreadsheet]
	    table $w $mb 1,0 -columnspan 100 -anchor w -fill x

	    set_table_buttons $db

	    button $w.quit \
		-text Quit \
		-activebackground red \
		-background red \
		-fg white \
		-command "destroy ."

	    table $w $w.quit 25,0 -columnspan 100 -anchor e
	    wm resizable $Window($database) no no

	}
    } else { 
	if { ! [ info exists Window([lindex $db 0])] } { 
	    set Window([lindex $db 0]) .noSuchWindow
	}
	set_Tables $db
	set schema [dbquery $db dbSCHEMA_NAME]
	if { [info exists Schema] } {
	    if { $schema != $Schema } { 
		puts stderr "All databases from the command line must share the same schema."
		puts stderr "  This command line has both $Schema and $schema"
		puts stderr "  Please run dbe separately on each database."
		exit
	    }
	} else { 
	    set Schema $schema 
	}

	catch "pfgetarr Defaults %${Pf}#$schema"
	set tablename [dbquery $db dbTABLE_NAME]
	if {[catch {dbquery $db dbRECORD_COUNT} nrecords]} { 
	    tkdialog .a "Can't open $tablename" 
	    return 
	} elseif { $nrecords == 0 } {
	    set reply [tkdialog .a "No rows" "Table $tablename has no records\nCreate new table?" Yes Cancel]
	    if { $reply != 0 } { 
		quit_when_dead .a
		return
	    } else { 
		newtable "$db" $tablename
	    }
	} else { 
	    set oldw dummy
	    if { $args == "" } {
		set w [show_table $db ]
	    } else { 
		global Request Record
		eval set Request($oldw) $args
		set Record($oldw) 0 
		set w [show_table $db $oldw]
	    }
	    global Nrecords
	    set Nrecords($w) $nrecords
	}
    }
}

